@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
- * 2012, 2013, 2014, 2015, 2016, 2018, 2019, 2020
+ * 2012, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021
  * Martin Lambers <marlam@marlam.de>
  * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
@@ -417,6 +417,10 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     {
         printf("SCRAM-SHA-1 ");
     }
+    if (session->cap.flags & POP3_CAP_AUTH_SCRAM_SHA_256)
+    {
+        printf("SCRAM-SHA-256 ");
+    }
     if (session->cap.flags & POP3_CAP_AUTH_EXTERNAL)
     {
         printf("EXTERNAL ");
@@ -1073,9 +1077,10 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
         }
         free(uidl->uidv);
         uidl->uidv = NULL;
-        uidl->n = session->old_number;
-        if (uidl->n > 0)
+        uidl->n = 0;
+        if (session->old_number > 0 && session->msg_uid)
         {
+            uidl->n = session->old_number;
             uidl->uidv = xmalloc(uidl->n * sizeof(char *));
             j = 0;
             for (i = 0; i < session->total_number; i++)
@@ -1115,9 +1120,10 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
         }
         free(uidl->uidv);
         uidl->uidv = NULL;
-        uidl->n = session->old_number;
-        if (uidl->n > 0)
+        uidl->n = 0;
+        if (session->old_number > 0 && session->msg_uid)
         {
+            uidl->n = session->old_number;
             uidl->uidv = xmalloc(uidl->n * sizeof(char *));
             j = 0;
             for (i = 0; i < session->total_number; i++)
@@ -2190,6 +2196,10 @@ int main(int argc, char *argv[])
         {
             printf("scram-sha-1 ");
         }
+        if (pop3_client_supports_authmech("SCRAM-SHA-256"))
+        {
+            printf("scram-sha-256 ");
+        }
         if (pop3_client_supports_authmech("EXTERNAL"))
         {
             printf("external ");
@@ -2264,7 +2274,7 @@ int main(int argc, char *argv[])
             free(conffile);
         }
         printf("\n");
-        printf(_("Copyright (C) 2020 Martin Lambers and others.\n"
+        printf(_("Copyright (C) 2021 Martin Lambers and others.\n"
                     "This is free software.  You may redistribute copies of "
                         "it under the terms of\n"
                     "the GNU General Public License "
